==> py22-snmp-3.1.3.tbz: lib/python2.2/site-packages/pysnmp/mapping/udp/role.py <==
"""
   Single-session, blocking network I/O classes.

   Copyright 1999-2002 by Ilya Etingof <ilya@glas.net>. See LICENSE for
   details.
"""
import socket, select
from pysnmp.mapping.udp import error

class manager:
    """Network client: send data item to server and receive a response
    """
    def __init__(self, agent=(None, 0), iface=('0.0.0.0', 0)):
        # Initialize defaults
        self.agent = agent
        self.iface = iface
        self.socket = None
        self.timeout = 1.0
        self.retries = 3

    def __del__(self):
        """Close socket on object termination
        """
        try:
            self.close()

        except error.TransportError:
            pass
        
    def get_socket(self):
        """
           get_socket() -> socket

           Return socket object previously created with open() method.
        """
        return self.socket

    def open(self):
        """
           open()
           
           Initialize transport layer (UDP socket) to be used
           for further communication with server.
        """
        try:
            self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

        except socket.error, why:
            raise error.NetworkError('socket() error: ' + str(why))

        # Bind to specific interface on client machine
        try:
            self.socket.bind(self.iface)

        except socket.error, why:
            raise error.NetworkError('bind() error: %s: %s' % (self.iface, why))

        # Connect to default destination if given
        if self.agent[0] is not None and self.agent[1] != 0:
            try:
                self.socket.connect(self.agent)

            except socket.error, why:
                raise error.NetworkError('connect() error: %s: %s' %
                                         (self.agent, why))

        return self.socket

    def send(self, request, dst=None):
        """
           send(req[, dst])
           
           Send "req" message (string) to server by address specified on
           object creation or by "dst" address given in socket module 
           notation.
        """
        # Make sure the connection is established, open it otherwise
        if not self.socket:
            self.open()

        try:
            if dst:
                self.socket.sendto(request, dst)
            else:    
                self.socket.send(request)

        except socket.error, why:
            raise error.NetworkError('send() error: ' + str(why))

    def read(self):
        """
           read() -> (message, src)
           
           Read data from the socket (assuming there's some data ready
           for reading), return a tuple of response message (as string)
           and source address 'src' (in socket module notation).
        """   
        # Make sure the connection exists
        if not self.socket:
            raise error.NetworkError('Socket not initialized')

        try:
            (message, src) = self.socket.recvfrom(65536)

        except socket.error, why:
            raise error.NetworkError('recv() error: ' + str(why))

        return (message, src)
        
    def receive(self):
        """
           receive() -> (message, src)
           
           Wait for incoming data from network or timeout (and return
           a tuple of None's).

           Return a tuple of received data item (as string) and source address
           'src' (in socket module notation).
        """
        # Make sure the connection exists
        if not self.socket:
            raise error.NetworkError('Socket not initialized')

        # Initialize sockets map
        r, w, x = [self.socket], [], []

        # Wait for response
        r, w, x = select.select(r, w, x, self.timeout)

        # Timeout occurred?
        if r:
            return self.read()

        # Return nothing on timeout
        return(None, None)

    def send_and_receive(self, message, dst=None):
        """
           send_and_receive(data[, dst]) -> (data, src)
           
           Send data item to remote entity by address specified on object 
           creation or 'dst' address and receive a data item in response
           or timeout (and raise NoResponse exception).

           Return a tuple of data item (as string) and source address
           'src' (in socket module notation).
        """
        # Initialize retries counter
        retries = 0

        # Send request till response or retry counter hits the limit
        while retries < self.retries:
            # Send a request
            self.send(message, dst)

            # Wait for response
            (response, src) = self.receive()

            # See if it's succeeded
            if response:
                return(response, src)

            # Otherwise, try it again
            retries = retries + 1

        # No answer, raise an exception
        raise error.NoResponseError('No response arrived before timeout')

    def close(self):
        """
           close()
           
           Terminate communication with remote server.
        """
        # See if it's opened
        if self.socket:
            try:
                self.socket.close()

            except socket.error, why:
                raise error.NetworkError('close() error: ' + str(why))

            # Initialize it to None to indicate it's closed
            self.socket = None  

class agent:
    """Network client: receive requests, send back responses
    """
    def __init__(self, (cb_fun, cb_ctx)=(None, None),
                 ifaces=[('0.0.0.0', 161)]):
        # Block on select() waiting for request by default
        self.timeout = None
        
        # Initialize defaults
        self.ifaces = ifaces
        self.socket = None

        # Store callback information
        self.cb_fun = cb_fun
        self.cb_ctx = cb_ctx
        
    def __del__(self):
        """Close socket on object termination
        """
        try:
            self.close()

        except error.TransportError:
            pass

    def get_socket(self):
        """
           get_socket() -> socket

           Return socket object previously created with open() method.
        """
        return self.socket

    def open(self):
        """
           open()
           
           Initialize transport internals to be used for further
           communication with client.
        """
        try:
            self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

        except socket.error, why:
            raise error.NetworkError('socket() error: ' + str(why))

        # Bind to specific interfaces at server machine
        for iface in self.ifaces:
            try:
                self.socket.bind(iface)

            except socket.error, why:
                raise error.NetworkError('bind() error: %s: %s' % (iface, why))

        return self.socket

    def send(self, message, dst):
        """
           send(rsp, dst)
           
           Send response message (given as string) to client process
           by 'dst' address given in socket module notation.
        """
        # Make sure the connection is established, open it otherwise
        if not self.socket:
            raise error.NetworkError('Socket not initialized')

        try:
            self.socket.sendto(message, dst)
                
        except socket.error, why:
            raise error.NetworkError('send() error: ' + str(why))

    def read(self):
        """
           read() -> (req, src)
           
           Read data from the socket (assuming there's some data ready
           for reading), return a tuple of request (as string) and
           source address 'src' (in socket module notation).
        """   
        # Make sure the connection exists
        if not self.socket:
            raise error.NetworkError('Socket not initialized')

        try:
            (message, peer) = self.socket.recvfrom(65536)

        except socket.error, why:
            raise error.NetworkError('recvfrom() error: ' + str(why))

        return (message, peer)
        
    def receive(self):
        """
           receive() -> (req, src)
           
           Wait for and receive request message from remote process
           or timeout.

           Return a tuple of request message (as string) and source address
           'src' (in socket module notaton).
        """
        # Attempt to initialize transport stuff
        if not self.socket:
            self.open()
            
        # Initialize sockets map
        r, w, x = [ self.socket ], [], []

        # Wait for response
        r, w, x = select.select(r, w, x, self.timeout)

        # Timeout occurred?
        if r:
            return self.read()

        raise error.IdleTimeoutError('No request arrived before timeout')
    
    def receive_and_send(self, (cb_fun, cb_ctx)=(None, None)):
        """
           receive_and_send((cb_fun, cb_ctx))
           
           Wait for request from a client process or timeout (and raise
           IdleTimeoutError exception), pass request to the callback function
           to build a response, send response back to client process.
        """
        if cb_fun is None:
            (cb_fun, cb_ctx) = (self.cb_fun, self.cb_ctx)
            
        if not callable(cb_fun):
            raise error.BadArgumentError('Non-callable callback function')

        while 1:
            # Wait for request to come
            (request, src) = self.receive()

            if not request:
                raise error.IdleTimeoutError('No request arrived before timeout')

            # Invoke callback function
            (response, dst) = cb_fun(self, cb_ctx, (request, src))

            # Send a response if any
            if (response):
                # Reply back by either source address or alternative
                # destination whenever given
                if dst:
                    self.send(response, dst)
                else:
                    self.send(response, src)

    def close(self):
        """
           close()
           
           Close UDP socket used for communication with client.
        """
        # See if it's opened
        if self.socket:
            try:
                self.socket.close()

            except socket.error, why:
                raise error.NetworkError('close() error: ' + str(why))

            # Initialize it to None to indicate it's closed
            self.socket = None

==> py22-snmp-3.3.2.tbz: lib/python2.2/site-packages/pysnmp/mapping/udp/role.py <==
"""
   Single-session, blocking network I/O classes.

   Copyright 1999-2002 by Ilya Etingof <ilya@glas.net>. See LICENSE for
   details.
"""
import socket, select
from pysnmp.mapping.udp import error

class manager:
    """Network client: send data item to server and receive a response
    """
    def __init__(self, agent=(None, 0), iface=('0.0.0.0', 0)):
        # Initialize defaults
        self.agent = agent
        self.iface = iface
        self.socket = None
        self.timeout = 1.0
        self.retries = 3

    def __del__(self):
        """Close socket on object termination
        """
        try:
            self.close()

        except error.TransportError:
            pass
        
    def get_socket(self):
        """
           get_socket() -> socket

           Return socket object previously created with open() method.
        """
        return self.socket

    def open(self):
        """
           open()
           
           Initialize transport layer (UDP socket) to be used
           for further communication with server.
        """
        try:
            self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

        except socket.error, why:
            raise error.NetworkError('socket() error: ' + str(why))

        # Bind to specific interface on client machine
        try:
            self.socket.bind(self.iface)

        except socket.error, why:
            raise error.NetworkError('bind() error: %s: %s' % (self.iface, why))

        # Connect to default destination if given
        if self.agent[0] is not None and self.agent[1] != 0:
            try:
                self.socket.connect(self.agent)

            except socket.error, why:
                raise error.NetworkError('connect() error: %s: %s' %
                                         (self.agent, why))

        return self.socket

    def send(self, request, dst=(None, 0)):
        """
           send(req[, dst])
           
           Send "req" message (string) to server by address specified on
           object creation or by "dst" address given in socket module 
           notation.
        """
        # Make sure the connection is established, open it otherwise
        if not self.socket:
            self.open()

        try:
            if dst[0] is not None and dst[1] != 0:
                self.socket.sendto(request, dst)
            else:    
                self.socket.send(request)

        except socket.error, why:
            raise error.NetworkError('send() error: ' + str(why))

    def read(self):
        """
           read() -> (message, src)
           
           Read data from the socket (assuming there's some data ready
           for reading), return a tuple of response message (as string)
           and source address 'src' (in socket module notation).
        """   
        # Make sure the connection exists
        if not self.socket:
            raise error.NetworkError('Socket not initialized')

        try:
            (message, src) = self.socket.recvfrom(65536)

        except socket.error, why:
            raise error.NetworkError('recv() error: ' + str(why))

        return (message, src)
        
    def receive(self):
        """
           receive() -> (message, src)
           
           Wait for incoming data from network or timeout (and return
           a tuple of None's).

           Return a tuple of received data item (as string) and source address
           'src' (in socket module notation).
        """
        # Make sure the connection exists
        if not self.socket:
            raise error.NetworkError('Socket not initialized')

        # Initialize sockets map
        r, w, x = [self.socket], [], []

        # Wait for response
        r, w, x = select.select(r, w, x, self.timeout)

        # Timeout occurred?
        if r:
            return self.read()

        # No answer, raise an exception
        raise error.NoResponseError('No response arrived before timeout')

    def send_and_receive(self, message, dst=(None, 0), cb_fun=None):
        """
           send_and_receive(data[, dst]) -> (data, src)
           
           Send data item to remote entity by address specified on object 
           creation or 'dst' address and receive a data item in response
           or timeout (and raise NoResponse exception).

           Return a tuple of data item (as string) and source address
           'src' (in socket module notation).
        """
        # Initialize retries counter
        retries = 0

        # Send request till response or retry counter hits the limit
        while retries < self.retries:
            # Send a request
            self.send(message, dst)

            # Wait for response
            while 1:
                try:
                    (response, src) = self.receive()

                except error.NoResponseError:
                    # Give it another try
                    break

                # Got some response
                if callable(cb_fun):
                    if cb_fun(response, src):
                        # Response is good
                        return (response, src)
                    else:
                        continue
                else:
                    # Response is not verified
                    return (response, src)

            # Otherwise, try it again
            retries = retries + 1

        # No answer, raise an exception
        raise error.NoResponseError('No response arrived in %d secs * %d retries' % (self.timeout, self.retries))

    def close(self):
        """
           close()
           
           Terminate communication with remote server.
        """
        # See if it's opened
        if self.socket:
            try:
                self.socket.close()

            except socket.error, why:
                raise error.NetworkError('close() error: ' + str(why))

            # Initialize it to None to indicate it's closed
            self.socket = None  

class agent:
    """Network client: receive requests, send back responses
    """
    def __init__(self, (cb_fun, cb_ctx)=(None, None),
                 ifaces=[('0.0.0.0', 161)]):
        # Block on select() waiting for request by default
        self.timeout = None
        
        # Initialize defaults
        self.ifaces = ifaces
        self.socket = None

        # Store callback information
        self.cb_fun = cb_fun
        self.cb_ctx = cb_ctx
        
    def __del__(self):
        """Close socket on object termination
        """
        try:
            self.close()

        except error.TransportError:
            pass

    def get_socket(self):
        """
           get_socket() -> socket

           Return socket object previously created with open() method.
        """
        return self.socket

    def open(self):
        """
           open()
           
           Initialize transport internals to be used for further
           communication with client.
        """
        try:
            self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

        except socket.error, why:
            raise error.NetworkError('socket() error: ' + str(why))

        # Bind to specific interfaces at server machine
        for iface in self.ifaces:
            try:
                self.socket.bind(iface)

            except socket.error, why:
                raise error.NetworkError('bind() error: %s: %s' % (iface, why))

        return self.socket

    def send(self, message, dst):
        """
           send(rsp, dst)
           
           Send response message (given as string) to client process
           by 'dst' address given in socket module notation.
        """
        # Make sure the connection is established, open it otherwise
        if not self.socket:
            raise error.NetworkError('Socket not initialized')

        try:
            self.socket.sendto(message, dst)
                
        except socket.error, why:
            raise error.NetworkError('send() error: ' + str(why))

    def read(self):
        """
           read() -> (req, src)
           
           Read data from the socket (assuming there's some data ready
           for reading), return a tuple of request (as string) and
           source address 'src' (in socket module notation).
        """   
        # Make sure the connection exists
        if not self.socket:
            raise error.NetworkError('Socket not initialized')

        try:
            (message, peer) = self.socket.recvfrom(65536)

        except socket.error, why:
            raise error.NetworkError('recvfrom() error: ' + str(why))

        return (message, peer)
        
    def receive(self):
        """
           receive() -> (req, src)
           
           Wait for and receive request message from remote process
           or timeout.

           Return a tuple of request message (as string) and source address
           'src' (in socket module notaton).
        """
        # Attempt to initialize transport stuff
        if not self.socket:
            self.open()
            
        # Initialize sockets map
        r, w, x = [ self.socket ], [], []

        # Wait for response
        r, w, x = select.select(r, w, x, self.timeout)

        # Timeout occurred?
        if r:
            return self.read()

        raise error.IdleTimeoutError('No request arrived before timeout')
    
    def receive_and_send(self, (cb_fun, cb_ctx)=(None, None)):
        """
           receive_and_send((cb_fun, cb_ctx))
           
           Wait for request from a client process or timeout (and raise
           IdleTimeoutError exception), pass request to the callback function
           to build a response, send response back to client process.
        """
        if cb_fun is None:
            (cb_fun, cb_ctx) = (self.cb_fun, self.cb_ctx)
            
        if not callable(cb_fun):
            raise error.BadArgumentError('Non-callable callback function')

        while 1:
            # Wait for request to come
            (request, src) = self.receive()

            if not request:
                raise error.IdleTimeoutError('No request arrived before timeout')

            # Invoke callback function
            (response, dst) = cb_fun(self, cb_ctx, (request, src))

            # Send a response if any
            if (response):
                # Reply back by either source address or alternative
                # destination whenever given
                if dst:
                    self.send(response, dst)
                else:
                    self.send(response, src)

    def close(self):
        """
           close()
           
           Close UDP socket used for communication with client.
        """
        # See if it's opened
        if self.socket:
            try:
                self.socket.close()

            except socket.error, why:
                raise error.NetworkError('close() error: ' + str(why))

            # Initialize it to None to indicate it's closed
            self.socket = None
